@@ -4,10 +4,10 @@ import nanoget.utils as ut
 import pandas as pd
 import sys
 import pysam
-import nanomath
 import re
 from Bio import SeqIO
 import concurrent.futures as cfutures
+from itertools import repeat
 
 
 def process_summary(summaryfile, **kwargs):
@@ -128,7 +128,7 @@ def process_ubam(bam, **kwargs):
         samfile = pysam.AlignmentFile(bam, "rb", check_sq=False)
         logging.info("Nanoget: No index for bam file could be found, created index.")
     datadf = pd.DataFrame(
-        data=[(read.query_name, nanomath.ave_qual(read.query_qualities), read.query_length)
+        data=[(read.query_name, ut.ave_qual(read.query_qualities), read.query_length)
               for read in samfile.fetch(until_eof=True)],
         columns=["readIDs", "quals", "lengths"]) \
         .dropna(axis='columns', how='all') \
@@ -154,20 +154,29 @@ def process_bam(bam, **kwargs):
     logging.info("Nanoget: Starting to collect statistics from bam file {}.".format(bam))
     samfile = check_bam(bam)
     chromosomes = samfile.references
-    if len(chromosomes) > 100:
-        params = [(bam, None)]
-        logging.info("Nanoget: lots of contigs (>100), not running in separate processes")
-    else:
-        params = zip([bam] * len(chromosomes), chromosomes)
-    with cfutures.ProcessPoolExecutor(max_workers=kwargs["threads"]) as executor:
+    if len(chromosomes) > 100 or kwargs["huge"]:
+        logging.info("Nanoget: lots of contigs (>100) or --huge, not running in separate processes")
         datadf = pd.DataFrame(
-            data=[res for sublist in executor.map(extract_from_bam, params) for res in sublist],
+            data=extract_from_bam(bam, None, kwargs["keep_supp"]),
             columns=["readIDs", "quals", "aligned_quals", "lengths",
                      "aligned_lengths", "mapQ", "percentIdentity"]) \
             .dropna(axis='columns', how='all') \
             .dropna(axis='index', how='any')
-    logging.info("Nanoget: bam {} contains {} primary alignments.".format(
-        bam, datadf["lengths"].size))
+
+    else:
+        unit = chromosomes
+        with cfutures.ProcessPoolExecutor(max_workers=kwargs["threads"]) as executor:
+            datadf = pd.DataFrame(
+                data=[res for sublist in executor.map(extract_from_bam,
+                                                      repeat(bam),
+                                                      unit,
+                                                      repeat(kwargs["keep_supp"]))
+                      for res in sublist],
+                columns=["readIDs", "quals", "aligned_quals", "lengths",
+                         "aligned_lengths", "mapQ", "percentIdentity"]) \
+                .dropna(axis='columns', how='all') \
+                .dropna(axis='index', how='any')
+    logging.info(f"Nanoget: bam {bam} contains {datadf['lengths'].size} primary alignments.")
     return ut.reduce_memory_usage(datadf)
 
 
@@ -187,20 +196,25 @@ def process_cram(cram, **kwargs):
     logging.info("Nanoget: Starting to collect statistics from cram file {}.".format(cram))
     samfile = check_bam(cram, samtype="cram")
     chromosomes = samfile.references
-    params = zip([cram] * len(chromosomes), chromosomes)
+    if len(chromosomes) > 100:
+        unit = [None]
+        logging.info("Nanoget: lots of contigs (>100), not running in separate processes")
+    else:
+        unit = chromosomes
     with cfutures.ProcessPoolExecutor(max_workers=kwargs["threads"]) as executor:
         datadf = pd.DataFrame(
-            data=[res for sublist in executor.map(extract_from_bam, params) for res in sublist],
+            data=[res for sublist in executor.map(extract_from_bam,
+                                                  repeat(cram), unit, repeat(kwargs["keep_supp"]))
+                  for res in sublist],
             columns=["readIDs", "quals", "aligned_quals", "lengths",
                      "aligned_lengths", "mapQ", "percentIdentity"]) \
             .dropna(axis='columns', how='all') \
             .dropna(axis='index', how='any')
-    logging.info("Nanoget: cram {} contains {} primary alignments.".format(
-        cram, datadf["lengths"].size))
+    logging.info(f"Nanoget: cram {cram} contains {datadf['lengths'].size} primary alignments.")
     return ut.reduce_memory_usage(datadf)
 
 
-def extract_from_bam(params):
+def extract_from_bam(bam, chromosome, keep_supplementary=True):
     """Extracts metrics from bam.
 
     Worker function per chromosome
@@ -212,18 +226,29 @@ def extract_from_bam(params):
     -mapping qualities
     -edit distances to the reference genome scaled by read length
     """
-    bam, chromosome = params
     samfile = pysam.AlignmentFile(bam, "rb")
-    return [
-        (read.query_name,
-         nanomath.ave_qual(read.query_qualities),
-         nanomath.ave_qual(read.query_alignment_qualities),
-         read.query_length,
-         read.query_alignment_length,
-         read.mapping_quality,
-         get_pID(read))
-        for read in samfile.fetch(reference=chromosome, multiple_iterators=True)
-        if not read.is_secondary and not read.is_unmapped]
+    if keep_supplementary:
+        return [
+            (read.query_name,
+             ut.ave_qual(read.query_qualities),
+             ut.ave_qual(read.query_alignment_qualities),
+             read.query_length,
+             read.query_alignment_length,
+             read.mapping_quality,
+             get_pID(read))
+            for read in samfile.fetch(reference=chromosome, multiple_iterators=True)
+            if not read.is_secondary and not read.is_unmapped]
+    else:
+        return [
+            (read.query_name,
+             ut.ave_qual(read.query_qualities),
+             ut.ave_qual(read.query_alignment_qualities),
+             read.query_length,
+             read.query_alignment_length,
+             read.mapping_quality,
+             get_pID(read))
+            for read in samfile.fetch(reference=chromosome, multiple_iterators=True)
+            if not read.is_secondary and not read.is_unmapped and not read.is_supplementary]
 
 
 def get_pID(read):
@@ -310,7 +335,7 @@ def extract_from_fastq(fq):
     Return average quality and read length
     """
     for rec in SeqIO.parse(fq, "fastq"):
-        yield nanomath.ave_qual(rec.letter_annotations["phred_quality"]), len(rec)
+        yield ut.ave_qual(rec.letter_annotations["phred_quality"]), len(rec)
 
 
 def stream_fastq_full(fastq, threads):
@@ -336,8 +361,8 @@ def extract_all_from_fastq(rec):
     """
     return (rec.id,
             len(rec),
-            nanomath.ave_qual(rec.letter_annotations["phred_quality"]),
-            nanomath.median_qual(rec.letter_annotations["phred_quality"]))
+            ut.ave_qual(rec.letter_annotations["phred_quality"]),
+            None)
 
 
 def info_to_dict(info):
@@ -364,7 +389,7 @@ def process_fastq_rich(fastq, **kwargs):
         try:
             read_info = info_to_dict(record.description)
             res.append(
-                (nanomath.ave_qual(record.letter_annotations["phred_quality"]),
+                (ut.ave_qual(record.letter_annotations["phred_quality"]),
                  len(record),
                  read_info["ch"],
                  read_info["start_time"],
